/*
 milix Class Library v1.1
 Blowfish encryption algorithm

 FILE  : blowfish.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2003 milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 The BLOWFISH encryption algorithm, considered as the most compact
 and secure block-encryption algorithm [2000].
 Remember that the length of the data to be encrypted must be divided
 by 8 or else the algorithm leaves the last (len%8) bytes unchanged.
*/

#ifndef _mcl_blowfish_h
#define _mcl_blowfish_h

#if !defined(_mcl_defs_h)
#include "defs.h"
#endif

// BEGIN NAMESPACE MCL
namespace mcl {

class blowfish
{
 public:
	// Constructor, destructor
	blowfish();
	~blowfish();

	// Initialize the engine with the key
	void      init(const unsigned char* key, unsigned char keylen);

	// You can pass the same pointer in in and out arguments.
	unsigned  encrypt(const unsigned char* in, unsigned char* out, unsigned len);
	void      decrypt(const unsigned char* in, unsigned char* out, unsigned len);

    // function used to determine a divideable by 8 length
    static unsigned length8(unsigned len) {
	   unsigned val = len % 8;
	   if (val != 0) len += (8-val);
	   return len;
    }
    
  private:
	unsigned* parray;
	unsigned  (*SBoxes)[256];
	void      blowfish_encipher(unsigned* xl, unsigned* xr);
	void      blowfish_decipher(unsigned* xl, unsigned* xr);
};

// END NAMESPACE MCL
};

#endif   // _mcl_crypt_h
