/*
 milix Class Library v1.1
 Expression evaluator

 FILE  : expreval.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2003 milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 Numerical and boolean evaluator. Supports variables.
*/

#ifndef _mcl_expreval_h
#define _mcl_expreval_h

#if !defined(_mcl_defs_h)
#include "defs.h"
#endif

// BEGIN NAMESPACE MCL
namespace mcl {

class expreval
{
 public:
	// Constructor, destructor
	expreval();
	virtual ~expreval() {}

	// Member functions
	int matherr() { return ec; }
	double calculate(const char* expr, ...);

 protected:
	enum op_t {
		op_none,
		op_neg,
		op_not,
		op_mul, op_div, op_and, op_pow,
		op_add, op_sub, op_or, op_xor,
		op_lt, op_gt, op_le, op_ge, op_eq, op_ne,
		op_lp, op_rp,
	};
	enum _arg {
		arg_end,
		arg_val,
		arg_op,
		arg_var,
	};
	struct arg_t
	{
		_arg what;
		union {
			double val;
			op_t   op;
		} arg;
	};

	// Private methods
	int compile(const char* expr);
	static int is_oper(const char* f);
	static int priority(int op);
	void LINK_OP(op_t);
	void LINK_VAL(double);

	// Private data
	int    ec;
	arg_t* compiled;
	int    tokens;
};

// [] INLINES []

inline void expreval::LINK_OP(op_t OP)
{
	compiled[tokens].what = arg_op; 
	compiled[tokens].arg.op = OP;
	tokens++;
}

inline void expreval::LINK_VAL(double VAL)
{ 
	compiled[tokens].what = arg_val; 
	compiled[tokens].arg.val = VAL;
	tokens++;
}

// END NAMESPACE MCL
};

#endif  // _mcl_expreval_h
