/*
 milix Class Library v1.1
 MD5 hash algorithm

 FILE  : md5.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2003 milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 Implementation of the MD5 message-digest algorithm.
 MD5 developed by Ronald L. Rivest of M.I.T
 
 Usage
 ~~~~~
 md5 context;
 context.update(block, vlock_len);
 context.finalise();
 constext.hex_digest(hex);
*/

#ifndef _mcl_md5_h
#define _mcl_md5_h

#if !defined(_mcl_defs_h)
#include "defs.h"
#endif

// BEGIN NAMESPACE MCL
namespace mcl {

class md5
{
 public:
	// Constructors
	md5();
 	md5(unsigned char* string); // digest string, finalize
	md5(int fh);                // digest file, finalize

	// Member functions
	void           update(unsigned char* input, unsigned len);
	void           update(int fh);
	void           finalize();

    // digest as a 16-byte binary array
	unsigned char* raw_digest(unsigned char* buf=0);
	// digest as a 33-byte ascii-hex string
	char*          hex_digest(char* hexstr=0);

 private:
	// Private methods
	void           init();
	void           transform(unsigned char* buf);
	static void    encode(unsigned char* dest, unsigned* src, unsigned len);
	static void    decode(unsigned* dest, unsigned char* src, unsigned len);
	static inline  unsigned rotate_left(unsigned x, int n);
	static inline  unsigned F(unsigned x, unsigned y, unsigned z);
	static inline  unsigned G(unsigned x, unsigned y, unsigned z);
	static inline  unsigned H(unsigned x, unsigned y, unsigned z);
	static inline  unsigned I(unsigned x, unsigned y, unsigned z);
	static inline  void     FF(unsigned& a, unsigned b, unsigned c, unsigned d,
							   unsigned x, unsigned s, unsigned ac);
	static inline  void     GG(unsigned& a, unsigned b, unsigned c, unsigned d,
							   unsigned x, unsigned s, unsigned ac);
	static inline  void     HH(unsigned& a, unsigned b, unsigned c, unsigned d,
							   unsigned x, unsigned s, unsigned ac);
	static inline  void     II(unsigned& a, unsigned b, unsigned c, unsigned d,
							   unsigned x, unsigned s, unsigned ac);

	// Private data
	unsigned      m_state[4];
	unsigned      m_count[2];
	unsigned char m_buffer[64];
	unsigned char m_digest[16];
	int           m_finalized;
};

// END NAMESPACE MCL
};

#endif   // _mcl_md5_h

