/*
 milix Class Library v1.1
 Stacks and queues.

 FILE  : staque.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2003 milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 Stack (LIFO) and Queue (FIFO) definitions and implementation.
 Usage:
 stack - stack<int>, stack<MyClass>, ...
 queue - queue>int>, queue<MyClass>, ...
*/

#ifndef _mcl_staque_h
#define _mcl_staque_h

#if !defined(_mcl_defs_h)
#include "defs.h"
#endif

// BEGIN NAMESPACE MCL
namespace mcl {

/////////////////////////////////////////////////
// Stack
// ~~~~~

template <class T>
class stack
{
 public:
	// Constructor, Destructor
	stack();
	~stack();

 public:
	// Member functions
	long  length() const { return count; }
	long  size() const { return count; }
	bool  empty() const { return bool(count == 0); }
	bool  push(const T&);
	T     pop();
	T&    top() { return h->data; }

 private:
	struct node_t {
		T       data;
		node_t* next;
	};

	// Private data members
	node_t* h;
	long    count;
};


/////////////////////////////////////////////////
// Queue
// ~~~~~

template <class T>
class queue
{
 public:
	 // Constructor, Destructor
	queue();
	~queue();

	// Member functions
	long  length() const { return count; }
	long  size() const { return count; }
	bool  empty() const { return bool(count == 0); }
	bool  push(const T&);
	T     pop();
	T&    top() { return front->data; }
	T&    bottom() { return tail->data; }

 private:
	struct node_t {
		T       data;
		node_t* next;
	};

	// Private data members
	node_t* front;
	node_t* tail;
	long    count;
};


/////////////////////////////////////////////////
// IMPLEMENTATION
/////////////////////////////////////////////////

template <class T>
stack<T>::stack()
{
	count = 0;
	h     = NULL;
}

template <class T>
stack<T>::~stack()
{
	node_t* n;
	while (h) {
		n = h->next;
		delete h;
		h = n;
	}
}

template <class T>
bool stack<T>::push(const T& obj)
{
	node_t* t;

	if ((t = new node_t) == 0) return false;
	t->data = obj;
	t->next = NULL;
	if (!h) {
        h = t;
    } else {
		t->next = h;
		h = t;
	}
	count++;
	return true;
}

template <class T>
T stack<T>::pop()
{
	T ret;
	node_t* t;

	if (!h) return NULL;
	ret = h->data;
	t = h;
	h = h->next;
	delete t;
	count--;
	return ret;
}

template <class T>
queue<T>::queue()
{
	front = tail = NULL;
	count = 0;
}

template <class T>
queue<T>::~queue()
{
	while (front) {
		tail = front;
		delete front;
		front = tail;
	}
}

template <class T>
bool queue<T>::push(const T& obj)
{
	node_t* t;

	if ((t = new node_t) == 0) return false;
	t->data = obj;
	t->next = NULL;
	if (!front) {
        front = tail = t;
    } else {
		tail->next = t;
		tail = t;
	}
	count++;
	return true;
}

template <class T>
T queue<T>::pop()
{
	T ret;
	node_t* t;
	
	if (!front) return ret;
	t = front;
	ret = t->data;
	front = front->next;
	delete t;
	count--;
	return ret;
}

// END NAMESPACE MCL
};

#endif   // _mcl_staque_h
