/*
 milix Class Library v1.1
 String convertion utilities

 FILE  : stri18n.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2003 milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 String convertion utilities
*/

#ifndef _mcl_stri18n_h
#define _mcl_stri18n_h

// BEGIN NAMESPACE MCL
namespace mcl {

// codepages
#define MCL_CP_ISO7          1
#define MCL_CP_1253          2
#define MCL_CP_737           3
#define MCL_CP_869           4
#define MCL_CP_LATGREEK      5
#define MCL_CP_ISOGREEK      MCL_CP_ISO7
#define MCL_CP_WINGREEK      MCL_CP_1253
#define MCL_CP_DOSGREEK      MCL_CP_737
#define MCL_CP_DOSGREEK2     MCL_CP_869

// functions that operate per character and can be used in strfunc()
int to_upper(int c, int codepage);
int to_lower(int c, int codepage);
int to_unaccent(int c, int codepage);
int to_unaccentupper(int c, int codepage);
int to_unaccentlower(int c, int codepage);

// functions for codepages ISO7 and 1253 (to speedup convertions)
int to_iso7upper(int c);
int to_iso7lower(int c);
int to_iso7unaccent(int c);
int to_iso7unaccentupper(int c);
int to_iso7unaccentlower(int c);
int to_1253upper(int c);
int to_1253lower(int c);
int to_1253unaccent(int c);
int to_1253unaccentupper(int c);
int to_1253unaccentlower(int c);

// functions that operate per string
char*          translate(const char* src, int scp, char* dst, int dcp);
char*          strfunc(char* str, int (*cnv)(int));
char*          strfunc(char* str, int arg2, int (*cnv)(int, int));
int            strxcmp(const char* a, const char* b, int (*cnv)(int));
int            strxcmp(const char* a, const char* b, int arg2, int (*cnv)(int,int));
int            strnxcmp(const char* a, const char* b, int (*cnv)(int));
int            strnxcmp(const char* a, const char* b, int arg2, int (*cnv)(int,int));
char*          fixfinal_s(char* str, int codepage);
unsigned short axtous(const char*);
unsigned int   axtou(const char*);
unsigned long  axtoul(const char*);
char*          strinscommas(char* str, char comma=',');
char*          strrmcommas(char* str, char comma=',');

// END NAMESPACE MCL
};

#endif   // _mcl_stri18n_h
