/*
 milix Class Library v1.1
 Text tokenizer

 FILE  : token.h
 AUTHOR: Anastasios Milikas (aka 'milix'). e-mail: milix@milix.net

 Copyright (c) 1999-2004 by milix
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 Overview
 ~~~~~~~~
 Splits the contents of a text file in tokens for further
 processing.

 Notes
 ~~~~~
 For single-line comments use set_comment("# ")
 THIS CLASS USES DLLIST.H
*/

#ifndef _mcl_token_h
#define _mcl_token_h

#if !defined(_mcl_dllist_h)
#include "dllist.h"
#endif

// BEGIN NAMESPCE MCL
namespace mcl {

// tokenizer ////////////////////////////////////
/////////////////////////////////////////////////

class tokenizer
{
 public:
	// Constructor, Destructor
	tokenizer();
	virtual ~tokenizer();

	// Member functions
	int length() const { return m_list.length(); }
	int size() const { return length(); }
	int set_separators(const char* separ);
	int set_sticky_separators(const char* sticky);
	int set_string_delimeters(const char* delim, bool include=true);
	int set_comment(const char* rem);
	int token(char* text, long n);

	// Virtual methods
	virtual int   compare(const char*, const char*);
	virtual int   set_buffer(const char* text, long textsz);
	virtual int   read_buffer(int fh, long filesz);
	virtual int   read_buffer_part(int fh, long filesz);
	virtual int   read_buffer_frame(int fh, long beg, long end);
	virtual int   preprocess(bool rmcomments=true, bool rmdefines=true);
	virtual bool  istext(long pos);
	virtual int   goto_label(const char* label, long from);
	virtual int   get_text(const char* label, char* ret, long from=0);
	virtual int   get_int16(const char* label, short* ret, long from=0);
	virtual int   get_int32(const char* label, int* ret, long from=0);
	virtual int   get_double(const char* label, double* ret, long from=0);

    // Operators
    void* item(long pos) { return (void*)m_list[pos]; }
	void* operator [](long pos) { return item(pos); }

 protected:
	// Private data
	char*         m_separ;
	char*         m_sticky;
	char*         m_rem;
	char*         m_delim;
	bool          m_incdelim;
	long          m_ofs;
	long          m_fofs;
	long          m_buffsz;
	char*         m_buff;
	dllist<char*> m_list;
	void _kill_list_entries();
};

// END NAMESPACE MCL
};

#endif   // _mcl_token_h
