/****************************************************************************
 stri18n.cpp

 milix Class Library
 Author: milix (milix@milix.net)
 Date  : 1999-2003

 Description
 ~~~~~~~~~~~
 String utilities implementation
*****************************************************************************/

#include <string.h>
#include <ctype.h>
#include "../include/stri18n.h"

using namespace mcl;

// codepages
#include "_i18n.h"

inline static unsigned char LATUNICODE(unsigned char* table, unsigned char letter)
{
    register int i;
    int len = (int)table[1]*2;
    table += 2;
    for (i=0; i<len; i += 2) {
        if (table[i] == letter) return table[i+1];
    }
    return 0;
}

inline static unsigned char UNICODE(unsigned char* table, unsigned char letter)
{
    unsigned char f = table[0];
    unsigned char n = table[((unsigned)table[1]-1)<<1];
    if (letter < f || letter > n) return 0;
    table += 2;
    return table[(((unsigned)letter-f)<<1)+1];
}

inline static unsigned char LETTER(unsigned char* table, unsigned char unicode)
{
    register int i;
    int len = (int)(((unsigned)table[1])<<1);
    table += 3;
    for (i=0; i<len; i+=2) {
        if (table[i] == unicode) return table[i-1];
    }
    return 0;
}

inline static unsigned char U_UPPERCASE(unsigned char u)
{
    if (u>=0xb1 && u<=0xc9) u-=0x20;
    else if (u==0xac) u=0x86;  // a'
    else if (u==0xad) u=0x88; // e'
    else if (u==0xae) u=0x89; // h'
    else if (u==0xaf) u=0x8a; // i'
    else if (u==0x90||u==0xca) u=0xaa; // i dialitika and tonos
    else if (u==0xcd) u=0x8e; // u'
    else if (u==0xb0||u==0xcb) u=0xab; // u wuth dialitika and tonos
    else if (u==0xcc) u=0x8c; // o'
    else if (u==0xce) u=0x8f; // w'
    return u;
}

inline static unsigned char U_LOWERCASE(unsigned char u)
{
    if (u>=0x91 && u<=0xa9) u+=0x20;
    else if (u==0x86) u=0xac; // a'
    else if (u==0x88) u=0xad; // e'
    else if (u==0x89) u=0xae; // h'
    else if (u==0x8a) u=0xaf; // i'
    else if (u==0xaa) u=0xca; // i dialitika
    else if (u==0x8e) u=0xcd; // u'
    else if (u==0xab) u=0xcb; // u wuth dialitika
    else if (u==0x8c) u=0xcc; // o'
    else if (u==0x8f) u=0xce; // w'
    return u;
}

inline static unsigned char U_UNACCENT(unsigned char u)
{
    if (u==0xac) u=0xb1; // alpha
    else if (u==0xad) u=0xb5; // epsilon
    else if (u==0xae) u=0xb7; // eta
    else if (u==0xcc) u=0xbf; // omicron
    else if (u==0xce) u=0xc9; // omega
    else if (u==0x90||u==0xaf||u==0xca) u=0xb9; // iota
    else if (u==0xb0||u==0xcb||u==0xcd) u=0xc5; // upsilon
    else if (u==0x86) u=0x91; // ALPHA
    else if (u==0x88) u=0x95; // EPSILON
    else if (u==0x89) u=0x97; // ETA
    else if (u==0x8c) u=0x9f; // OMICRON
    else if (u==0x8f) u=0xa9; // OMEGA
    else if (u==0x8a||u==0xaa) u=0x99; // IOTA
    else if (u==0x8e||u==0xab) u=0xa5; // UPSILON
    return u;
}

// to_upper
// upercases the character c
int mcl::to_upper(int c, int codepage)
{
    unsigned char uc = (unsigned char)c;
    switch (codepage) {
    case MCL_CP_ISO7:
        uc = UNICODE(CPISO7, uc);
        if (uc==0) return toupper(c);
        uc = LETTER(CPISO7, U_UPPERCASE(uc));
        break;
    case MCL_CP_1253:
        uc = UNICODE(CP1253, uc);
        if (uc==0) return toupper(c);
        uc = LETTER(CP1253, U_UPPERCASE(uc));
        break;
    case MCL_CP_737:
        uc = UNICODE(CP737, uc);
        if (uc==0) return toupper(c);
        uc = LETTER(CP737, U_UPPERCASE(uc));
        break;
    case MCL_CP_869:
        uc = UNICODE(CP869, uc);
        if (uc==0) return toupper(c);
        uc = LETTER(CP869, U_UPPERCASE(uc));
        break;
    default:
        uc=toupper(uc);
        break;
    }
    return uc;
}

// to_lower
// lowercases the character c
int mcl::to_lower(int c, int codepage)
{
    unsigned char uc = (unsigned char)c;
    switch (codepage) {
    case MCL_CP_ISO7:
        uc = UNICODE(CPISO7, uc);
        if (uc==0) return tolower(c);
        uc = LETTER(CPISO7, U_LOWERCASE(uc));
        break;
    case MCL_CP_1253:
        uc = UNICODE(CP1253, uc);
        if (uc==0) return tolower(c);
        uc = LETTER(CP1253, U_LOWERCASE(uc));
        break;
    case MCL_CP_737:
        uc = UNICODE(CP737, uc);
        if (uc==0) return tolower(c);
        uc = LETTER(CP737, U_LOWERCASE(uc));
        break;
    case MCL_CP_869:
        uc = UNICODE(CP869, uc);
        if (uc==0) return tolower(c);
        uc = LETTER(CP869, U_LOWERCASE(uc));
        break;
    default:
        uc=tolower(uc);
        break;
    }
    return uc;
}

// to_unaccent
// remove the accent mark if character c is accented
int mcl::to_unaccent(int c, int codepage)
{
    unsigned char uc = (unsigned char)c;
    switch (codepage) {
    case MCL_CP_ISO7:
        uc = UNICODE(CPISO7, uc);
        if (uc==0) return (unsigned char) c;
        uc = LETTER(CPISO7, U_UNACCENT(uc));
        break;
    case MCL_CP_1253:
        uc = UNICODE(CP1253, uc);
        if (uc==0) return (unsigned char) c;
        uc = LETTER(CP1253, U_UNACCENT(uc));
        break;
    case MCL_CP_737:
        uc = UNICODE(CP737, uc);
        if (uc==0) return (unsigned char) c;
        uc = LETTER(CP737, U_UNACCENT(uc));
        break;
    case MCL_CP_869:
        uc = UNICODE(CP869, uc);
        if (uc==0) return (unsigned char) c;
        uc = LETTER(CP869, U_UNACCENT(uc));
        break;
    }
    return uc;
}

// to_unaccentlower
// remove the accent mark if character c is accented and lowercases it
int to_unaccentlower(int c, int codepage)
{
    unsigned char uc = (unsigned char)c;
    switch (codepage) {
    case MCL_CP_ISO7: uc = UNICODE(CPISO7, uc); break;
    case MCL_CP_1253: uc = UNICODE(CP1253, uc); break;
    case MCL_CP_737: uc = UNICODE(CP737, uc); break;
    case MCL_CP_869: uc = UNICODE(CP869, uc); break;
    }
    if (uc==0) return tolower((unsigned char)c);
	uc = U_UNACCENT(uc);
	uc = U_LOWERCASE(uc);
    switch (codepage) {
    case MCL_CP_ISO7: uc = LETTER(CPISO7, uc); break;
    case MCL_CP_1253: uc = LETTER(CP1253, uc); break;
    case MCL_CP_737: uc = LETTER(CP737, uc); break;
    case MCL_CP_869: uc = LETTER(CP869, uc); break;
    }
    return uc;
}

// to_unaccentupper
// remove the accent mark if character c is accented and uppercases it
int mcl::to_unaccentupper(int c, int codepage)
{
    unsigned char uc = (unsigned char)c;
    switch (codepage) {
    case MCL_CP_ISO7: uc = UNICODE(CPISO7, uc); break;
    case MCL_CP_1253: uc = UNICODE(CP1253, uc); break;
    case MCL_CP_737: uc = UNICODE(CP737, uc); break;
    case MCL_CP_869: uc = UNICODE(CP869, uc); break;
    }
    if (uc==0) return toupper((unsigned char)c);
	uc = U_UNACCENT(uc);
	uc = U_UPPERCASE(uc);
    switch (codepage) {
    case MCL_CP_ISO7: uc = LETTER(CPISO7, uc); break;
    case MCL_CP_1253: uc = LETTER(CP1253, uc); break;
    case MCL_CP_737: uc = LETTER(CP737, uc); break;
    case MCL_CP_869: uc = LETTER(CP869, uc); break;
    }
    return uc;
}

// Codepage ISO7
int to_iso7upper(int c)
{
    unsigned char uc;
    uc = UNICODE(CPISO7, (unsigned char)c);
    if (uc==0) return toupper(c);
    return LETTER(CPISO7, U_UPPERCASE(uc));
}

int to_iso7lower(int c)
{
    unsigned char uc;
    uc = UNICODE(CPISO7, (unsigned char)c);
    if (uc==0) return tolower(c);
    return LETTER(CPISO7, U_LOWERCASE(uc));
}

int to_iso7unaccent(int c)
{
    unsigned char uc;
    uc = UNICODE(CPISO7, (unsigned char)c);
    if (uc==0) return (unsigned char) c;
    return LETTER(CPISO7, U_UNACCENT(uc));
}

int to_iso7unaccentupper(int c)
{
    unsigned char uc;
    uc = UNICODE(CPISO7, (unsigned char)c);
    if (uc==0) return toupper(c);
	uc = U_UNACCENT(uc);
	uc = U_UPPERCASE(uc);
    uc = LETTER(CPISO7, uc);
    return uc;
}

int to_iso7unaccentlower(int c)
{
    unsigned char uc;
    uc = UNICODE(CPISO7, (unsigned char)c);
    if (uc==0) return tolower(c);
	uc = U_UNACCENT(uc);
	uc = U_LOWERCASE(uc);
    uc = LETTER(CPISO7, uc);
    return uc;
}

// Codepage 1253 (Windows)
int to_1253upper(int c)
{
    unsigned char uc;
    uc = UNICODE(CP1253, (unsigned char)c);
    if (uc==0) return toupper(c);
    return LETTER(CP1253, U_UPPERCASE(uc));
}

int to_1253lower(int c)
{
    unsigned char uc;
    uc = UNICODE(CP1253, (unsigned char)c);
    if (uc==0) return tolower(c);
    return LETTER(CP1253, U_LOWERCASE(uc));
}

int to_1253unaccent(int c)
{
    unsigned char uc;
    uc = UNICODE(CP1253, (unsigned char)c);
    if (uc==0) return (unsigned char) c;
    return LETTER(CP1253, U_UNACCENT(uc));
}

int to_1253unaccentupper(int c)
{
    unsigned char uc;
    uc = UNICODE(CP1253, (unsigned char)c);
    if (uc==0) return toupper(c);
	uc = U_UNACCENT(uc);
	uc = U_UPPERCASE(uc);
    uc = LETTER(CP1253, uc);
    return uc;
}

int to_1253unaccentlower(int c)
{
    unsigned char uc;
    uc = UNICODE(CP1253, (unsigned char)c);
    if (uc==0) return tolower(c);
	uc = U_UNACCENT(uc);
	uc = U_LOWERCASE(uc);
    uc = LETTER(CP1253, uc);
    return uc;
}

// translate
// translates a string for one codepage to another
char* mcl::translate(const char* src, int scp, char* dst, int dcp)
{
    unsigned char u, *SCP, *DCP;
    register unsigned char *sp = (unsigned char*)src;
    register unsigned char *dp = (unsigned char*)dst;
    
    // set the codepage pointers
    switch (scp) {
        case MCL_CP_ISO7:     SCP = CPISO7;     break;
        case MCL_CP_1253:     SCP = CP1253;     break;
        case MCL_CP_737:      SCP = CP737;      break;
        case MCL_CP_869:      SCP = CP869;      break;
        case MCL_CP_LATGREEK: SCP = CPLATGREEK; break;
    };
    switch (dcp) {
        case MCL_CP_ISO7:     DCP = CPISO7;     break;
        case MCL_CP_1253:     DCP = CP1253;     break;
        case MCL_CP_737:      DCP = CP737;      break;
        case MCL_CP_869:      DCP = CP869;      break;
        case MCL_CP_LATGREEK: DCP = CPLATGREEK; break;
    };

    // check if want to translate from lattin greek
    if (scp == MCL_CP_LATGREEK) {
        while (*sp) {
            *dp = *sp;
            u = LATUNICODE(SCP, *sp);
            if (u) *dp = LETTER(DCP, u);
            sp++;
            dp++;
        }
        *dp = '\0';
        return dst;
    }
    
    // translate
    while (*sp) {
        *dp = *sp;
        u = UNICODE(SCP, *sp);
        if (u) *dp = LETTER(DCP, u);
        sp++;
        dp++;
    }
    *dp = '\0';
    
    return dst;
}

// strfunc
// performs the 'cvt' binary function to each character in the string
char* mcl::strfunc(char* dst, int (*cvt)(int))
{
	register char* p = dst;
	while ((*p = (char)(*cvt)(*p)) != 0) p++;
	return dst;
}

// strfunc
// performs the 'cvt' unary function to each character in the string
char* mcl::strfunc(char* dst, int arg2, int (*cvt)(int, int))
{
	register char* p = dst;
	while ((*p = (char)(*cvt)(*p, arg2)) != 0) p++;
	return dst;
}

// strxcmp
// performs a char by char comparision between two strings.
// the binary function 'cvt' is used before comparition between two chars
int mcl::strxcmp(const char* a, const char* b, int (*cvt)(int))
{
	int cmp = 0;
	unsigned char ac, bc;

    if (!cvt) return strcmp(a, b);
	while (cmp==0) {
		ac = (unsigned char)(*cvt)(*a);
		bc = (unsigned char)(*cvt)(*b);
        cmp = ac - bc;
		a++;
		b++;
		if (!*a && !*b) break;
	}
	return cmp;
}

// strxcmp
// performs a char by char comparision between two strings.
// the unary function 'cvt' is used before comparition between two chars
int mcl::strxcmp(const char* a, const char* b, int arg2, int (*cvt)(int,int))
{
	int cmp = 0;
	char ac, bc;

    if (!cvt) return strcmp(a, b);
	while (cmp==0) {
		ac = (char)(*cvt)(*a, arg2);
		bc = (char)(*cvt)(*b, arg2);
		cmp = ac - bc;
		a++;
		b++;
		if (!*a && !*b) break;
	}
	return cmp;
}

// strnxcmp
// performs a char by char comparision between two strings.
// the binary function 'cvt' is used before comparition between two chars
int strnxcmp(const char* a, const char* b, int n, int (*cvt)(int))
{
	int cmp = 0;
	unsigned char ac, bc;

    if (!cvt) return strncmp(a, b, n);
	while (cmp==0 && n>0) {
		ac = (unsigned char)(*cvt)((unsigned char)*a);
		bc = (unsigned char)(*cvt)((unsigned char)*b);
        cmp = ac - bc;
		a++;
		b++;
		n--;
		if (!*a && !*b) break;
	}
	return cmp;
}

// strnxcmp
// performs a char by char comparision between two strings.
// the binary function 'cvt' is used before comparition between two chars
int strnxcmp(const char* a, const char* b, int n, int arg2, int (*cvt)(int,int))
{
	int cmp = 0;
	unsigned char ac, bc;

    if (!cvt) return strncmp(a, b, n);
	while (cmp==0 && n>0) {
		ac = (unsigned char)(*cvt)((unsigned char)*a, arg2);
		bc = (unsigned char)(*cvt)((unsigned char)*b, arg2);
        cmp = ac - bc;
		a++;
		b++;
		n--;
		if (!*a && !*b) break;
	}
	return cmp;
}

// fixfinal_s
// fixes the problem with greek final s, when appears like mid-s in the end of words
char* mcl::fixfinal_s(char* str, int codepage)
{
    register char* p = str;
    register unsigned char u, *CP;
    switch (codepage) {
        case MCL_CP_LATGREEK: return str;
        case MCL_CP_ISO7: CP = CPISO7; break;
        case MCL_CP_1253: CP = CP1253; break;
        case MCL_CP_737:  CP = CP737;  break;
        case MCL_CP_869:  CP = CP869;  break;
    };
    while (*p) {
        u = UNICODE(CP, (unsigned char)*p);
        if (u == 0xc3) {
            switch (*(p+1)) {
            case ' ': case '\t': case '\n': case '\r': case '\0': 
			case '.': case ',': case ';': case '?': case '_':
                *p = LETTER(CP, 0xc2);
                break;
            }
        }
        p++;
    }
    return str;
}

// axtous
// converts a string that contains a hex value to unsigned short
unsigned short mcl::axtous(const char* s)
{
	register char *p;
	unsigned short sum = 0, pow = 1;

	for (p = (char*)s ; isxdigit(*p) ; p++);
	for (p-- ; p >= s ; p--) {
		if (*p >= '0' && *p <= '9') sum += ((*p - '0')*pow);
		else if (*p >= 'a' && *p <= 'f') sum += ((*p - 'W')*pow);
		else if (*p >= 'A' && *p <= 'F') sum += ((*p - '7')*pow);
		else break;
		pow <<= 4;
	}
	return sum;
}

// axtou
// converts a string that contains a hex value to unsigned int
unsigned int mcl::axtou(const char* s)
{
	register char *p;
	unsigned int sum = 0, pow = 1;

	for (p = (char*)s ; isxdigit(*p) ; p++);
	for (p-- ; p >= s ; p--) {
		if (*p >= '0' && *p <= '9') sum += ((*p - '0')*pow);
		else if (*p >= 'a' && *p <= 'f') sum += ((*p - 'W')*pow);
		else if (*p >= 'A' && *p <= 'F') sum += ((*p - '7')*pow);
		else break;
		pow <<= 4;
	}
	return sum;
}

// axtoul
// converts a string that contains a hex value to unsigned long
unsigned long mcl::axtoul(const char* s)
{
	register char *p;
	unsigned long sum = 0L, pow = 1L;

	for (p = (char*)s ; isxdigit(*p) ; p++);
	for (p-- ; p >= s ; p--)
	{
		if (*p >= '0' && *p <= '9') sum += ((*p - '0')*pow);
		else if (*p >= 'a' && *p <= 'f') sum += ((*p - 'W')*pow);
		else if (*p >= 'A' && *p <= 'F') sum += ((*p - '7')*pow);
		else break;
		pow <<= 4;
	}
	return sum;
}

// strinscommas
// formats a string using commas (eg. 1,000,000)
char* mcl::strinscommas(char* str, char comma)
{
	char t, *p;
    int len, digits;
	register int i, f;

    // find the string's length
	len = strlen(str);

    // reverse string
	for (i=0 ; i<len ; i++) {
		t = str[i];
		str[i] = str[len-i-1];
		str[len-i-1] = t;
	}
	
	// add commas
	p = strchr(str, '.');
	if (!p) {
		p = str;
		f = 0;
	} else {
		p++;
		f = (int)(p - str);
	}
	digits = i = 0;
	while (*p) {
        if (isdigit(*p)) digits++;
        if (digits == 3 && isdigit(*(p+1))) {
            digits = 0;
            i++;
            memmove(str+f+i+1, str+f+i, strlen(str+f)-i+1);
            *(str+f+i) = comma;
            len++;
			p++;
		}
		i++;
		p++;
	}
	
	// reverse string
	for (i=0 ; i<len ; i++) {
		t = str[i];
		str[i] = str[len-i-1];
		str[len-i-1] = t;
	}

	return str;
}

// strrmcommas
// remove commas from a string
char* mcl::strrmcommas(char* str, char comma)
{
	char* p;
	register int i;
	int len = strlen(str);

	for (p = strchr(str, comma) ; p ; p=strchr(str, comma)) {
		for (i=(int)(p-str); i<len ; i++) str[i] = str[i+1];
		len--;
	}
	return str;
}

